#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "circbuf.h"
 
/*!
 * Circular Buffer (Keep one slot open)
 * Adapted from Wikipedia 
 * Buffer and elements must be powers of two.
 */
 
#define BUFFER_SIZE  0x1800/sizeof(SAMPLE)
#define BUFFER_MASK  BUFFER_SIZE-1
#define NUM_OF_ELEMS   (BUFFER_SIZE-1)
 
/**< Circular Buffer Types */

uint16_t writePointer; /**< write pointer */
uint16_t readPointer;  /**< read pointer */
uint16_t size;         /**< size of circular buffer */
uint16_t mask;         /**< bit Mask of circular buffer size*/
SAMPLE circbuf[BUFFER_SIZE];
 
void CircBufferInit(void)
{
  size = BUFFER_SIZE;
  mask = size-1; 
  writePointer = 0;
  readPointer  = 0;
}
 
inline int CircBufferIsFull(void)
{
	return (((writePointer + 1)& mask) == readPointer); 
}
 
inline int CircBufferIsEmpty(void)
{
	return( readPointer == writePointer);
}
 
int CircBufferEnque(SAMPLE* psample)
{	
	
//	Do not need interrupt management here.
//  This is ONLY called from ISR's which have already turned off interrupts.
//	uint8_t sreg;  
//	sreg = SREG;
//	cli();

	uint8_t isFull = CircBufferIsFull();
	if(!isFull) 
	{
//		circbuf[writePointer].flag = psample->flag;
		circbuf[writePointer].value = psample->value;
//		circbuf[writePointer].timerH = psample->timerH;
		circbuf[writePointer].timerL = psample->timerL;
		writePointer++;
		writePointer &= mask;
	}
//	SREG = sreg;
    return isFull;
}

int CircBufferDeque(SAMPLE* psample)
{
	uint8_t sreg; // Need interrupt management. Called from foreground.
	sreg = SREG;
	cli();
    int isEmpty =  CircBufferIsEmpty();
	if(!isEmpty) 
	{
//		psample->flag 	= circbuf[readPointer].flag;
		psample->value 	= circbuf[readPointer].value;
//		psample->timerH	= circbuf[readPointer].timerH;
		psample->timerL	= circbuf[readPointer].timerL;
		readPointer++;
		readPointer &= mask;
	}
		
	SREG = sreg;
	return(isEmpty);
}
 

 
