#!/usr/local/bin/python

# File: seranl.py
from Tkinter import *
import sys
import usb.core
import usb.util
import threading
import time
import gc

VSIZE = 384
HORSIZE = 512
NCHAN = 6
SAMPLECLOCK = 16000000/64
USB_BUFSIZE = 512
BAUD = 9600
VENDORID = 0x03eb
PRODUCTID =	0x204F
class UartState:
	idle 	= 0
	start 	= 1
	bits	= 2
	parity  = 3
	stop	= 4
	searchidle = 5

rollover = 0x80
overrun  = 0x40
	
class Channel:
	def __init__(self,chnum,id):
		self.enabled = 0
		self.channelNumber = chnum
		self.chanId = id
		self.chanMask = 1 << chnum
	def setEnabled(self,enb):
		self.enabled = enb
	def getEnabled(self):
		return self.enabled
	def setChannelNumber(self,chn):
		self.channelNumber = chn
	def getChannelNumber(self):
		return self.channelNumber
	def getChannelId(self):
		return self.chanId
	def getChanMask(self):
		return self.chanMask

class DecodeSlice:
	def __init__(self,chanArray):
		self.chans = chanArray
		self.outstr = ''
	def getDecodeString(self):
		return self.outstr
	def Write(self, fileName):
		f = open(fileName,'w')
		f.write(self.outstr)
		f.close
	def decode(self,decslice,ch,baud,nbits,parity,stopbits,decodeTime):
		del decodeTime[:]
		self.outstr = ''
		l = len(decslice)
		if l == 0:
			print "No captured elements"
		elif self.chans[ch].enabled == 1:
#			print "Enabled channel %d"%ch
			tcap = decslice[0]
			tmin = tcap[1]
			tscan = tmin
			bitTime  = SAMPLECLOCK/baud # sample clocks per bit
			chn = self.chans[ch]
			mask = chn.getChanMask()
#			print "Decode channel mask = %x\n"%mask
			l = len(decslice)
			endcap = decslice[l-1]
			slctmax = endcap[1]
			slctmin = 0
			tmax = slctmax
			state = UartState.searchidle
			word = 0
			bitCount = 0
			startTime = 0	# all in sample clocks	
			wordTime = 0 
			lastcap = []
			timeMark = ("black",0)
			for cap in decslice:
				if state == UartState.searchidle: # looking for 1
					if (cap[0] & mask) != 0:
						state = UartState.idle
				elif state == UartState.idle: # looking for falling edge
					word = 0
					bitCount = 0
					startTime = 0	# all in sample clocks
					wordTime = 0 
					bitmask = 1
					timeMark = ("yellow",cap[1])	
					decodeTime.append(timeMark)
					if (cap[0] & mask) == 0:
						startTime = cap[1]
						wordTime = bitTime/2
						timeMark = ("red",startTime + wordTime)
						decodeTime.append(timeMark)
						state = UartState.start
						lastcap = cap
						continue
				if ((state != UartState.searchidle) and (state != UartState.idle)):
					while wordTime < (cap[1] - startTime):
						if state == UartState.start:
							timeMark = ("red",startTime+wordTime)
							state = UartState.bits
							wordTime += bitTime
						elif state == UartState.bits:
							if (lastcap[0] & mask) != 0 :
								word = word + bitmask
							timeMark = ("blue",startTime+wordTime)
							wordTime += bitTime
							bitCount += 1
							bitmask = bitmask * 2
							if bitCount == nbits:
								if parity != 'N':
									state = UartState.parity
								else:
									state = UartState.stop
						elif state == UartState.parity:
							# do parity check 
							timeMark = ("violet",startTime+wordTime)
							state = UartState.stop
							wordTime += bitTime
						elif state == UartState.stop:
							if (lastcap[0] & mask) != 0: 
								timeMark = ("orange",startTime+wordTime) # stop state
								self.outstr = self.outstr +  str(chr(word))								
								if (cap[0] & mask) == 0: # next sample is 0, start bit
									word = 0
									bitmask = 1
									bitCount = 0
									startTime = cap[1]	# all in sample clocks	
									wordTime = bitTime/2
									state = UartState.start
								else:
									wordTime += bitTime
									state = UartState.idle
									break
							else:
								timeMark = ("grey",startTime+wordTime) # missing stop bit!!!
								wordTime += bitTime

						decodeTime.append(timeMark)
					lastcap = cap

class Capture(threading.Thread):	
	def __init__(self):
		threading.Thread.__init__(self,name="Capture")
	# find our device
		self.dev = usb.core.find(idVendor=VENDORID, idProduct=PRODUCTID)

		# was it found?
		if self.dev is None:
			raise ValueError('Device not found')

		if self.dev.is_kernel_driver_active(0):
			try:
				self.dev.detach_kernel_driver(0)
			except usb.core.USBError as e:
				sys.exit("Could not detatch kernel driver: %s" % str(e))

		# set the active configuration. With no arguments, the first	
		# configuration will be the active one

		self.dev.set_configuration()
		self.dev.reset()
		sys.stdout.write(' Found Device \n.')
		for cfg in self.dev:
			sys.stdout.write(str(cfg.bConfigurationValue) + '\n')
		# get an endpoint instance
		self.cfg = self.dev[0]
		self.intf = self.cfg[(0,0)]
		self.ep = self.intf[0]

		self.captureStore = []
		self.filteredStore = []
		self.gocapture = 0

	def runCapture(self):
		self.start()
#		print "runCapture running"

	def stopCapture(self):
		self.gocapture = 0
		while self.gocapture >=0 :
			time.sleep(1)

	def getCaptureBuffer(self,buf):
		buf = self.captureStore

	def getFilteredStore(self,buf):
		buf = self.filteredStore

	def run(self):
		index = 0
		array = []
		captureLength = 0
		packetId = -1
		self.gocapture = 1
		gc.disable()
		while self.gocapture == 1:
			try:
				av = self.dev.read(self.ep.bEndpointAddress, USB_BUFSIZE, self.intf.bInterfaceNumber, 1000)
				array.append(av)
			except usb.core.USBError as e:
				print(" Capture Error %s "%e.args)
				break  

		for ar in array:
			index = 1
			for arrayval in ar:
				if index % 2 == 0 :
					deltaTime = ar[index -1] 
					self.captureStore.append( (ar[index-2],deltaTime) )
				index = index + 1

		gc.enable()
		self.filterCap(self.captureStore,self.filteredStore)
		index = 0
#		for cap in self.filteredStore:
#			print("%2x %4x"%cap),
#			index = index + 1
#			if (index) % 8 == 0 :
#				print 
#			if index  > 256 : 
#				break;

#		print "\nRuncapture Stopped"
		self.gocapture = -1

	def filterCap(self,inbuf,outbuf):
		time = 0
		lastcap = 0
		lastval = 0
		rollover = 0x100
		critical = 40
		for cap in inbuf:
			if (cap[0] & 0x80) == 0x80: #roll over	
				if (lastval & 0x80) == 0x80: # last was a roll over too...
					time = time + cap[1] + rollover - lastcap
				else: #critical zone
					newtime = cap[1] + rollover - lastcap
					if newtime > rollover:
						newtime -= rollover
					time = time + newtime
			elif cap[1] < lastcap : # must have rolled
				time = time + cap[1] + rollover - lastcap
			else:
				time = time + cap[1] - lastcap
			lastcap = cap[1]
			lastval = cap[0]
			outbuf.append((lastval,time))
			if (lastval & 0x40) == 0x40:
				print "Overrun ... %x %x"% (lastval,time)


	def getSlice(self,sliceExport,mintime,maxtime):
		cap0 = self.filteredStore[0]
		tmin = cap0[1] + mintime
		l = len(self.filteredStore)
		capend = self.filteredStore[l - 1]
		if maxtime == 0:
			maxtime = capend[1]
		elif maxtime > (capend[1] - cap0[1] ):
			maxtime = capend[1] - cap0[1]
		tmax = cap0[1] + maxtime
#		print "mintime %x maxtime %x filtMin %x filtMax %x tmin %x tmax %x" % (mintime,maxtime,cap0[1],capend[1],tmin,tmax)
		addedStart  = 1
		edgeCap = []
		for cap in self.filteredStore:
			if cap[1] < tmin:
				edgeCap = cap
				addedStart = 0
				continue
			elif cap[1] <= tmax:
				if addedStart == 0:
					tcap = (edgeCap[0],0)
					sliceExport.append(tcap)
					addedStart =1
				tcap = (cap[0],cap[1] - tmin)
				sliceExport.append(tcap)

class Gui(Tk):
	
	def __init__(self, top,hs,vs,chsz):
		self.run = 0
		self.tmin = 0
		self.tmax = 100
		self.zoomfactor = 1
		self.decodeTime = [("red",0)]
		self.frame = Frame(top,width=hs+chsz,height=vs+chsz)
		self.frame.grid(row=0,rowspan=(vs/chsz)+4,column=0,columnspan=(hs/chsz)+1)

		self.chanArray = []
		self.chanArray.append(Channel(0,'RCV'))
		self.chanArray[0].setEnabled(1)
		self.chanArray.append(Channel(1,'XMT'))
		self.chanArray.append(Channel(2,'CTS'))
		self.chanArray.append(Channel(3,'RTS'))
		self.chanArray.append(Channel(4,'DTR'))
		self.chanArray.append(Channel(5,'DSR'))
		
		index = 0
		self.buttons=[]
		self.enbButton = PhotoImage(master=top,file="enabled.gif")
		self.disButton = PhotoImage(master=top,file="disabled.gif")
#		print vs/chsz
#		print hs/chsz
		while index < vs/chsz:
			self.buttons.append(Button(self.frame, image=self.disButton))
			self.buttons[index].configure(command = lambda i = index: self.doEnable(i))
			self.buttons[index].grid(row = index, column = 0,pady=4)
			index = index + 1

		self.startButton = Button(self.frame, text = "Start", fg = "green",command = self.startCollecting)
		self.startButton.grid(row = index+2, column = 02,pady=20)
		self.stopButton = Button(self.frame, text = "Stop", fg="red",command = self.stopCollecting)
		self.stopButton.grid(row = index+2, column = 3,pady=10)
	
		self.leftendButton = Button(self.frame, text = "[",command = self.leftend)
		self.leftendButton.grid(row = index+2, column = 4,pady=10,sticky=E)
		self.scrollleftButton = Button(self.frame, text = "<",command = self.scrollleft)
		self.scrollleftButton.grid(row = index+2, column = 5,pady=10)
		self.scrollrightButton = Button(self.frame, text = ">",command = self.scrollright)
		self.scrollrightButton.grid(row = index+2, column = 6,pady=10)
		self.rightendButton = Button(self.frame, text = "]",command = self.rightend)
		self.rightendButton.grid(row = index+2, column = 7,pady=10)
		self.zoominButton = Button(self.frame, text = "+", command = self.zoomin)
		self.zoominButton.grid(row = index+2, column = 8, pady = 10,sticky=E)
		self.zoomoutButton = Button(self.frame, text = "-", command = self.zoomout)
		self.zoomoutButton.grid(row = index+2, column = 9, pady = 10)
		self.exitButton = Button(self.frame, text = "Exit",command = sys.exit)
		self.exitButton.grid(row = index+2, column = 10,pady=10,sticky=E)
		self.baudrate = Spinbox(width=8,values=(300, 1200,4800,9600,19200))
		self.baudrate.grid(row = index+4, column = 2,sticky=W)
		self.lbaudrate = Label(self.frame,text='      Baud Rate')
		self.lbaudrate.grid(row = index+4, column = 2, sticky=N)
		self.dataBits = Spinbox(width=5,values=(5,6,7,8))
		self.dataBits.grid(row = index+4, column = 4,pady=10,sticky=W)
		self.ldataBits = Label(self.frame,text='      Data Bits')
		self.ldataBits.grid(row = index+4, column = 4, sticky=N)
		self.parityBit = Spinbox(width=4,values=('N','E','O'))
		self.parityBit.grid(row = index+4, column = 5,pady=10,sticky=W)
		self.lparityBit = Label(self.frame,text='Parity')
		self.lparityBit.grid(row = index+4, column = 6, sticky=N)
		self.stopBits = Spinbox(width=5,values=(1,2))
		self.stopBits.grid(row = index+4, column = 6,pady=10,sticky=W)
		self.lstopBits = Label(self.frame,text='  Stop Bits')
		self.lstopBits.grid(row = index+4, column = 8, sticky=N)

	def updateButtonImages(self,vs,chsz):
		idx = 0
		while idx < vs/chsz:
			if self.chanArray[idx].getEnabled() == 0 :
				self.buttons[idx].configure(image = self.disButton)
			else:
				self.buttons[idx].configure(image = self.enbButton)
			idx = idx + 1
		self.baudSet()

	def startCollecting(self):
		self.capture = 0
		self.capture = Capture()
		if self.run == 0:
			self.run =1
			self.capture.runCapture()

	def stopCollecting(self):
		if self.run == 1:
			self.run = 0
#		print "stop capture"
		self.capture.stopCapture()
		self.drawSlice(0,0)
	def leftend(self):
		self.tmin = 0
		self.tmax = self.tmin + 512*self.zoomfactor
		self.drawSlice(self.tmin,self.tmax)
	def scrollleft(self):
		self.tmin = self.tmin - 480*self.zoomfactor
		if self.tmin < 0:
			self.tmin = 0
			self.tmax = self.tmin + 512*self.zoomfactor
		else:
			self.tmax = self.tmax - 480*self.zoomfactor
		self.drawSlice(self.tmin,self.tmax)
	def scrollright(self):
		self.tmin = self.tmin + 480*self.zoomfactor
		self.tmax = self.tmin + 512*self.zoomfactor
		self.drawSlice(self.tmin,self.tmax)
	def rightend(self):
		print " Not implemented"
	def zoomin(self):
		self.zoomfactor = self.zoomfactor / 2
		if self.zoomfactor < 1 :
			self.zoomfactor = 1
		self.tmax = self.tmin + 512*self.zoomfactor	
		self.drawSlice(self.tmin,self.tmax)
	def zoomout(self):
		self.zoomfactor = self.zoomfactor * 2
		self.tmax = self.tmin + 512*self.zoomfactor	
		self.drawSlice(self.tmin,self.tmax)
	def makeCanvas(self,w,h):
		self.canvas=Canvas(self.frame,width=w,height=h,bg="white")
		self.canvas.grid(row=0,rowspan=14,column=2,columnspan=18,sticky=N)

	def drawLine(self,x1,y1,x2,y2,color):
		self.canvas.create_line(x1,y1,x2,y2,width = 2,fill=color)
		p = (x1,y1,x2,y2)
#		print "%x %x %x %x" % p

	def drawText(self,x,y,txt):
		self.canvas.create_text(x,y,text=txt)

	def doEnable(self,idx):
		if self.chanArray[idx].getEnabled() == 0 :
			self.chanArray[idx].setEnabled(1)
			self.buttons[idx].configure(image = self.enbButton)
		else:
			self.buttons[idx].configure(image = self.disButton)
			self.chanArray[idx].setEnabled(0)

	def baudSet(self):
		return int(self.baudrate.get())

	def dataBitsSet(self):
		return int(self.dataBits.get())
	
	def parityBitSet(self):
		return self.parityBit.get()

	def stopBitsSet(self):
		return int(self.stopBits.get())
		
	def getFilteredStore(self):
		return self.capture.getFilteredStore()

	def printSlice(self,ts):
		print "Print Slice size %x" % len(ts)
		index = 0
		deltaT = 0
		deltas = []
		lastcap = ts[0]
		for cap in ts:
			val = cap[0]
			t   = cap[1]
			deltatuple = (val,t,deltaT)
			deltas.append(deltatuple)
			deltaT = cap[1] - lastcap[1]
			lastcap = cap

		deltaSorted = sorted(deltas, key=lambda d: d[2]) 
		for dt in deltaSorted:
			print "%2x %4x %3x" % dt,
			index = index +1
			if (index) % 4 == 0 :
				print

	def drawSlice(self,tmin,tmax):
		self.canvas.delete(ALL)
		if self.capture == 0:
			print "No capture buffer"
		else :
			tslice = []
			self.capture.getSlice(tslice,tmin,tmax)
			l = len(tslice)
			c = tslice[l-1]
			slctmax = c[1]
			slctmin = 0
			if tmax == 0:
				tmin = 0
				self.tmin = tmin
				tmax = slctmax
				self.tmax = tmax
			self.zoomfactor = (tmax-tmin)/HORSIZE 
			self.drawChannels(tslice,self.zoomfactor,self.chanArray,tmin,tmax)

	def drawChannels(self,mySlice,zoom,chans,tmin,tmax):
		ydwnshift = 10
		yoff = VSIZE/NCHAN
		lastcap = (0,0)
		first = 0
		yval = 0
		ybase = 0
		ych = 0
		pp = (yoff,zoom)
#		print "Yoff %x zoomfact %x" % pp
		for cap in mySlice:
#			print "%x %x "%cap,
			if first == 0:
				lastcap = cap
				first = 1
				continue
			for ch in chans:
				if ch.getEnabled() >0 :
					yval = yoff/2
					mask = ch.getChanMask()
					chn = ch.getChannelNumber()
					ybase = (yoff * chn) + ydwnshift
					if mask & lastcap[0]:
						yval = 0
					ych = (yval + ybase) 
					self.drawLine(lastcap[1]/zoom,ych,cap[1]/zoom,ych,"blue")
					if (lastcap[0] & mask) != (cap[0] & mask):
						if yval > 0:
							yval = 0
						else:
							yval = yoff/2
						self.drawLine(cap[1]/zoom,ych,cap[1]/zoom,yval+ybase,"green")
			lastcap = cap
#		if (lastcap[1]/zoom) < HORSIZE :
#			print "lastcap %x %x" % lastcap
#			self.drawline(lastcap[1]/zoom,ych,HORSIZE,ych,"blue")
		dec = DecodeSlice(chans)
		nameBase = 'Decode_'
		for ch in chans:
			chn = ch.getChannelNumber()
			ybase = (yoff * chn) + ydwnshift
			txbase = (yoff * chn) + yoff/3
			self.drawText(20,txbase,ch.getChannelId())
			if ch.getEnabled() > 0 :
				dec.decode(mySlice,chn,self.baudSet(),self.dataBitsSet(),self.parityBitSet(),self.stopBitsSet(),self.decodeTime)
				print dec.outstr
				fname = nameBase + str(chn) + '.txt'
				dec.Write(fname)
				self.displayDecodeSampleTimes(ybase,zoom,self.decodeTime)
				self.drawText(20,txbase,ch.getChannelId())
				
	def displayDecodeSampleTimes(self,yoff,zoom,decTimes):
#		print "Disp decode yoff = %d"%yoff
		if len(decTimes) > 1:
			first = decTimes[0]
			tmin = first[1]
			l = len(decTimes)
			last = decTimes[l-1]
			tmax = last[1]
			for t in decTimes:
				self.drawLine(t[1]/zoom,yoff,t[1]/zoom,yoff+20,t[0])

root = Tk()

gui = Gui(root,HORSIZE,VSIZE,64)
gui.updateButtonImages(HORSIZE,VSIZE)
gui.makeCanvas(HORSIZE,VSIZE)
gui.drawLine(1,1,HORSIZE,1,"black")
gui.drawLine(HORSIZE,1,HORSIZE,VSIZE,"black")
gui.drawLine(HORSIZE,VSIZE,1,VSIZE,"black")
gui.drawLine(1,VSIZE,1,1,"black")

i = 0
lines = []
nlines = 6
spacer = VSIZE/nlines
xstart = 1
xend = HORSIZE
ystart = spacer-10

root.mainloop()
